#include <proto.h>

#include "gtmenubar.h"
#include "gtmenu.h"
#include "gterror.h"
#include "gtcommon.h"
#include "gtconfig.h"

/************************************************************************\
*
* GtMenuBar class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init menubar.
*
\************************************************************************/
GtMenuBar *GtMkMenuBar(char *n,GtManager *p,GtTag *tgs)
{
	GtMenuBar *obj;
	
	if(!(obj=new GtMenuBar()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtMenuBar object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init menubar.
*
\************************************************************************/
GtMenuBar *GtMkMenuBar(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtMenuBar *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkMenuBar(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty menubar.
*
\************************************************************************/
GtMenuBar::GtMenuBar(void)
{
	frm=NULL;
	sub=NULL;
	sxp=0;
	syp=0;
	sxs=0;
	sys=0;
	ftype=GTF_OUT;
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtMenuBar::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtMenuBar")==0)
		return(TRUE);
	else
		return(GtGroup::IsSubClassOf(cn));
}

/************************************************************************\
*
* Handle menubar events.
*
\************************************************************************/
int GtMenuBar::Handle(XEvent *xe)
{
	int ret;

	if(!(flags & GTWF_MAPPED))
		return(GTHR_SEARCH);
	ret=GtGroup::Handle(xe);
	if(xe && (xe->type==ButtonRelease) && (xe->xbutton.button==GTDEF_MENUBUTTON))
		this->Popdown();
	if((xe->type!=Expose) && dsp->HasGrab(this))
		return(GTHR_USED);
	return(ret);
}

/************************************************************************\
*
* Set menubar event mask.
*
\************************************************************************/
void GtMenuBar::SetEMask(ulong em)
{
	GtGroup::SetEMask(em|ButtonReleaseMask|EnterWindowMask);
}

/************************************************************************\
*
* Notify popdown of sub menu.
*
\************************************************************************/
void GtMenuBar::NotifyA(ulong tag,void *arg)
{
	if(tag==GT_Popdown)
		this->PopdownSub();
	else
		GtGroup::NotifyA(tag,arg);
}

/************************************************************************\
*
* Popup sub menu.
*
\************************************************************************/
void GtMenuBar::PopupSub(GtWidget *from,GtMenu *menu,Time tm)
{
	long xp,yp,xs,ys;
	
	sub=menu;
	from->GetV(GT_XPos,&xp,GT_YPos,&yp,GT_XSize,&xs,GT_YSize,&ys,GT_DONE);
	sxp=xp;
	syp=yp;
	sxs=xs;
	sys=ys;
	dsp->SetGrab(this);
	menu->Popup(from,GTDEF_MENUBUTTON,tm);
}

/************************************************************************\
*
* Popdown sub menu.
*
\************************************************************************/
void GtMenuBar::PopdownSub(void)
{
	if(sub)
	{
		sub->Popdown();
		dsp->ClrGrab(this);
	}
}

/************************************************************************\
*
* Mouse menubar events.
*
\************************************************************************/
sbool GtMenuBar::Mouse(XEvent *xe)
{
	sbool ret;
	
	ret=GtGroup::Mouse(xe);
	if(xe)
	{
		switch(xe->type)
		{
			case ButtonRelease:
				if(xe->xbutton.button==GTDEF_MENUBUTTON)
				{
					ret=TRUE;
					this->Popdown();
				}
				break;
		}
	}
	return(ret);
}

/************************************************************************\
*
* Release all control from the menubar.
*
\************************************************************************/
void GtMenuBar::Popdown(void)
{
	if(sub)
	{
		sub->Popdown();
	}
	dsp->ClrGrab(this);
	if(frm)
		frm->NotifyA(GT_Popdown,this);
}
